#include <Servo.h>

Servo servo1; // Il servomotore

#define echoPinFrontale 2                            // Pin to receive echo pulse
#define trigPinFrontale 4                            // Pin to send trigger pulse
#define novanta 80 // è la posizione mediana del servomotore (90 gradi): per asimmetrie varie tale valore può essere diverso da 90
#define zero 0 // fine corsa destro (0 gradi) del servomotore
#define centottanta 2*novanta // fine corsa sinistro (180 gradi) del servomotore


#define echoPinTerra 13                            // Pin to receive echo pulse US verso terra
#define trigPinTerra 12                            // Pin to send trigger pulse US verso terra

int pot = 0; // il cursore del potenziometro collegato ad A0
int posizione; // La posizione del cursore del potenziometro

int pinDirA = 7; // Pin di direzione motore A (destro)
int pinDirB = 8; // Pin di direzione motore B (sinistro)
int pinPwmA = 3; // Pin PWM motore A (destro)
int pinPwmB = 11; // Pin PWM motore B (sinistro)
float raddrizzaFactor = 1.08; // Per compensare eventuali differenze nella velocità dei due motori (>1 avvantaggia il sinistro, <1 il destro)
#define Ts 100 // tempo tra una lettura US e la successiva
long oldTime = 0; // tempo ultima lettura 

int potenzaPWM = 80;

int distanzaLetta, distanzaLetta0, distanzaLetta179; // distanza letta dall'US frontale (in cm)
int distanzaTerra; // distanza letta dall'US puntato verso terra (in cm)
 
void setup()
 {
   // Tutti i pin dei due motori sono settati in output:
   pinMode( pinDirA,OUTPUT );
   pinMode( pinPwmA,OUTPUT );
   pinMode( pinDirB,OUTPUT );
   pinMode( pinPwmB,OUTPUT );

   // Per la comunicazione delle letture IR col serial monitor
   // Serial.begin(9600);   
   // Parte di comunicazione con sensore a ultrasuoni SRF04
   pinMode(echoPinFrontale, INPUT); // Il pin del segnale di eco del sensore US frontale
   pinMode(trigPinFrontale, OUTPUT); // Il pin del segnale di output del sensore US frontale
   pinMode(echoPinTerra, INPUT); // Il pin del segnale di eco del sensore US puntato verso terra
   pinMode(trigPinTerra, OUTPUT); // Il pin del segnale di output del sensore US puntato verso terra
   oldTime = millis();
   
   servo1.attach(6); // Colleghiamo virtualmente il motore al pin 6
   servo1.write(novanta); // sposta il motore della posizione gradi   
   delay(500);
 }
 
void loop()
 {

  posizione = analogRead(pot); // legge il potenziometro
  potenzaPWM = map(posizione,0,1023,0,150); // mappa da una scala da 0 a 1023 (la lettura analogica del potenziometro è a 1024 livelli) a una scala da 0 a 150 (il PWM arriva a 255 ma si è scelto di non andare mai al max)
   
   if (millis()-oldTime > Ts) // Una misura ogni Ts
   {   
     oldTime = millis();
     servo1.write(novanta); // sposta il servomotore in modo che l'US frontale punti in avanti
     delay(500);
     distanzaLetta = distanzaUS(echoPinFrontale,trigPinFrontale); // leggo la distanza libera davanti al robot
     distanzaTerra = distanzaUS(echoPinTerra,trigPinTerra); // leggo la distanza della superficie su cui il robot sta avanzando
     // Serial.println(distanzaLetta);
     // Serial.println(distanzaTerra);  
   }
   
   if (distanzaTerra > 10) // Ho davanti un precipizio
   {
     // Torno indietro
     digitalWrite(pinDirA,LOW);
     digitalWrite(pinDirB,LOW);
     analogWrite(pinPwmA,potenzaPWM);
     analogWrite(pinPwmB,int(potenzaPWM*raddrizzaFactor));
     delay(1500); // indietreggio per 1.5 sec
     distanzaLetta = 0; // ponendo distanzaLetta = 0 entro anche nel ciclo successivo: questo per fargli fare una curva dopo che ha indietreggiato
   }
     
   if (distanzaLetta < 25)
   {

     // mi fermo
     analogWrite(pinPwmA,0);
     analogWrite(pinPwmB,0);

     servo1.write(zero); // sposta il servomotore a zero gradi (guardo a destra)
     delay(500);
     distanzaLetta0 = distanzaUS(echoPinFrontale,trigPinFrontale); // e leggo quanto spazio libero ho a destra
 
     servo1.write(centottanta);  // sposta il servomotore a centottanta gradi (guardo a sinistra)
     delay(500); 
     distanzaLetta179 = distanzaUS(echoPinFrontale,trigPinFrontale); // e leggo quanto spazio libero ho a sinistra
     servo1.write(novanta); // riporto l'US a guardare in avanti
     delay(500);
     
     if (distanzaLetta0 > distanzaLetta179) // a destra è più libero e quindi giro a destra
     {
        digitalWrite(pinDirA,LOW);
        digitalWrite(pinDirB,HIGH);
     }
     else     // a sinistra è più libero e quindi giro a sinistra
     {
        digitalWrite(pinDirA,HIGH);
        digitalWrite(pinDirB,LOW);
     }

     analogWrite(pinPwmA,potenzaPWM);
     analogWrite(pinPwmB,int(potenzaPWM*raddrizzaFactor));
     delay(random(400,1000)); // giro per un certo tempo casuale
   }
   else  // non ci sono ostacoli davanti e posso andare dritto
   {
     digitalWrite( pinDirA,HIGH);
     digitalWrite( pinDirB,HIGH);
   
     analogWrite( pinPwmA,potenzaPWM);
     analogWrite( pinPwmB,int(potenzaPWM*raddrizzaFactor) );
   }
 }
 
 int distanzaUS(int echoPin, int trigPin) // funzione per leggere l'US
 {
   int impulsi;
   
   digitalWrite(trigPin, LOW);                   // Set the trigger pin to low for 2uS
   delayMicroseconds(2);
   digitalWrite(trigPin, HIGH);                  // Send a 10uS high to trigger ranging
   delayMicroseconds(10);
   digitalWrite(trigPin, LOW);                   // Send pin low again
   impulsi = pulseIn(echoPin, HIGH);            // Read in times pulse
   return int(impulsi/58);                        // Calculate distance from time of pulse
 }
   
